umask 022

P=python3
V=3.7.0
B=1

M=${V%%.*}
VM=${V%.*}
MM=${VM//./}

# rotate logs
for i in $(seq 9 -1 0); do [ -f package.log.$i ] && mv package.log.$i package.log.$(( i + 1 )); done
[ -f package.log ] && mv package.log package.log.0
exec > >(tee package.log) 2>&1

PREFIX=apps/Python$MM/
MAINTAINER=JuergenFischer

if ! [ -d "$OSGEO4W_REP" ]; then
	echo "OSGEO4W_REP not set"
	exit 1
fi

set -e
set -x

exetmpl() {
	local i=$1
	local pkg=$2
	local b=$(basename $i)
	local t=${PREFIX}Scripts/$b

	echo -e "textreplace -std -t ${t///\\}\r" >>$pkg-postinstall.bat
	echo -e "del ${t//\//\\\\}\r" >>$pkg-preremove.bat

	perl -pe "s#${PY//\\/\\\\}#\@osgeo4w\@\\\\apps\\\\Python$MM\\\\python.exe#i" $i >$b.tmpl
	chmod a+rx $b.tmpl
}

for a in x86_64 x86; do
	R=$OSGEO4W_REP/$a/release
	mkdir -p $R

	case "$a" in
	x86_64)
		PKG=python-$V-amd64.exe
		;;
	x86)
		PKG=python-$V.exe
		;;
	esac

	[ -f $PKG ] || curl https://www.python.org/ftp/python/$V/$PKG >$PKG
	chmod a+rx $PKG

	if ! [ -d $a ]; then
		#./$PKG /quiet /uninstall
		d=$(cygpath -w $PWD/$a)
		./$PKG /quiet TargetDir=${d//\\/\\\\} AssociateFiles=0 Shortcuts=0 SimpleInstall=1
	fi

	[ -d $a ]

	for p in $P-core $P-help $P-devel $P-test $P-tcltk $P-tools; do
		mkdir -p $R/$P/$p
		cp $a/LICENSE.txt $R/$P/$p/$p-$V-$B.txt
	done

	PIPV=$(sed -ne "s/^Version: //p" $a/Lib/site-packages/pip-*.dist-info/METADATA)
	STV=$(sed -ne "s/^Version: //p" $a/Lib/site-packages/setuptools-*.dist-info/METADATA)

	mkdir -p $R/$P/$P-pip $R/$P/$P-setuptools
	cp $a/LICENSE.txt $R/$P/$P-pip/$P-pip-$PIPV-$B.txt
	cp $a/LICENSE.txt $R/$P/$P-setuptools/$P-setuptools-$STV-$B.txt

	cat <<EOF >$R/$P/$P-core/setup.hint
sdesc: "Python core interpreter and runtime"
ldesc: "Python core interpreter and runtime"
maintainer: $MAINTAINER
category: Commandline_Utilities
requires: msvcrt2015 sqlite3
EOF

	PY=$(cygpath -aw $a/python.exe)

	cat <<EOF >tcltk.lst
$a/DLLs/_tkinter.pyd
$a/DLLs/tcl86t.dll
$a/DLLs/tk86t.dll
$a/Lib/idlelib
$a/tcl
EOF

	cat <<EOF >test.lst
$a/DLLs/_ctypes_test.pyd
$a/DLLs/_testbuffer.pyd
$a/DLLs/_testcapi.pyd
$a/DLLs/_testconsole.pyd
$a/DLLs/_testimportmultiple.pyd
$a/DLLs/_testmultiphase.pyd
$a/Lib/ctypes/test
$a/Lib/distutils/tests
$a/Lib/doctest.py
$a/Lib/idlelib/idle_test
$a/Lib/lib2to3/tests
$a/Lib/sqlite3/test
$a/Lib/test
$a/Lib/tkinter/test
$a/Lib/unittest/test
$a/tcl/tcl8/8.5/tcltest-2.4.1.tm
$a/Tools/scripts/run_tests.py
EOF

	cat <<EOF >pip.lst
./pip.exe.tmpl
./pip$M.exe.tmpl
./pip$VM.exe.tmpl
$a/Lib/site-packages/pip
$a/Lib/site-packages/pip-$PIPV.dist-info
EOF

	cat <<EOF >setuptools.lst
easy_install-$VM.exe.tmpl
easy_install.exe.tmpl
$a/Lib/site-packages/easy_install.py
$a/Lib/site-packages/setuptools
$a/Lib/site-packages/setuptools-$STV.dist-info
EOF

	cat <<EOF >tools.lst
$a/Tools
EOF

	cat <<EOF >preremove-cached.py
import importlib.util
import gzip
import os
import sys

cachedirs = {}
with gzip.open("{}/etc/setup/{}.lst.gz".format(os.environ['OSGEO4W_ROOT'], sys.argv[1])) as f:
    for py in f:
        py = py.decode("utf-8").strip()
        if py.endswith(".py"):
            try:
                pyc = importlib.util.cache_from_source(py)
                os.remove(pyc)
                print("Removed {}".format(pyc))
                cachedirs[ os.path.dirname(pyc) ] = 1
            except:
                pass

for cachedir in sorted(cachedirs.keys(), reverse=True):
    try:
        os.rmdir(cachedir)
        print("Removed directory {}".format(cachedir))
    except:
        pass
EOF

	#
	# core
	#

	cat <<EOF >py3_env.bat
SET PYTHONPATH=
SET PYTHONHOME=%OSGEO4W_ROOT%\\apps\\Python$MM
PATH %OSGEO4W_ROOT%\\apps\\Python$MM;%OSGEO4W_ROOT%\\apps\\Python$MM\\Scripts;%PATH%
EOF

	cat <<EOF >core-preremove.bat
call %OSGEO4W_ROOT%\\bin\\py3_env.bat
python -B %OSGEO4W_ROOT%\\apps\\Python$MM\\Scripts\\preremove-cached.py $P-core
EOF

	cp -a $a/python.exe python.exe
	cp -a $a/pythonw.exe pythonw.exe
	tar -cjf $R/$P/$P-core/$P-core-$V-$B.tar.bz2 \
		--xform "s,preremove-cached.py,${PREFIX}Scripts/preremove-cached.py," \
		--xform "s,core-preremove.bat,etc/preremove/$P-core.bat," \
		--xform "s,^$a/python.exe,${PREFIX}python.exe," \
		--xform "s,^$a/pythonw.exe,${PREFIX}pythonw.exe," \
		--xform "s,^python.exe,bin/python$M.exe," \
		--xform "s,^pythonw.exe,bin/pythonw$M.exe," \
		--xform "s,^$a/python$MM.dll,bin/python$MM.dll," \
		--xform "s,^$a/python$M.dll,bin/python$M.dll," \
		--xform "s,^$a/,$PREFIX," \
		--xform "s,py3_env.bat,bin/py3_env.bat," \
		--exclude "__pycache__" \
		--exclude $a/DLLs/sqlite3.dll \
		--exclude-from tcltk.lst \
		--exclude-from test.lst \
		--exclude-from pip.lst \
		--exclude-from setuptools.lst \
		--exclude-from tools.lst \
		preremove-cached.py \
		core-preremove.bat \
		py3_env.bat \
		$a/LICENSE.txt \
		$a/DLLs \
		$a/Lib \
		$a/python.exe \
		$a/pythonw.exe \
		python.exe \
		pythonw.exe \
		$a/python$MM.dll \
		$a/python$M.dll

	#
	# help
	#

	cat <<EOF >$R/$P/$P-help/setup.hint
sdesc: "Python documentation in a Windows compiled help file"
ldesc: "Python documentation in a Windows compiled help file"
maintainer: $MAINTAINER
category: Commandline_Utilities
requires: $P-core
external-source: $P-core
EOF

	tar -cjf $R/$P/$P-help/$P-help-$V-$B.tar.bz2 \
		--xform "s,$a/,$PREFIX," \
		$a/Doc/ \
		$a/NEWS.txt

	#
	# devel
	#


	cat <<EOF >$R/$P/$P-devel/setup.hint
sdesc: "Python library and header files"
ldesc: "Python library and header files"
maintainer: $MAINTAINER
category: Libs
requires: $P-core
external-source: $P-core
EOF

	tar -cjf $R/$P/$P-devel/$P-devel-$V-$B.tar.bz2 \
		--xform "s,$a/,$PREFIX," \
		$a/include \
		$a/libs

	#
	# test
	#

	cat <<EOF >$R/$P/$P-test/setup.hint
sdesc: "Python self tests"
ldesc: "Python self tests"
maintainer: $MAINTAINER
category: Libs
requires: $P-core
external-source: $P-core
EOF

	cat <<EOF >test-preremove.bat
call %OSGEO4W_ROOT%\\bin\\py3_env.bat
python -B %OSGEO4W_ROOT%\\apps\\Python$MM\\Scripts\\preremove-cached.py $P-test
EOF

	tar -cjf $R/$P/$P-test/$P-test-$V-$B.tar.bz2 \
		--xform "s,test-preremove.bat,etc/setup/$P-test.bat," \
		--xform "s,$a/,$PREFIX," \
		--exclude "__pycache__" \
		-T test.lst \
		test-preremove.bat

	#
	# tcltk & idle
	#

	cat <<EOF >$R/$P/$P-tcltk/setup.hint
sdesc: "Python Tkinter and IDLE"
ldesc: "Python Tkinter and IDLE"
maintainer: $MAINTAINER
category: Commandline_Utilities
requires: $P-core
external-source: $P-core
EOF

	cat <<EOF >tcltk-preremove.bat
call %OSGEO4W_ROOT%\\bin\\py3_env.bat
python -B %OSGEO4W_ROOT%\\apps\\Python$MM\\Scripts\\preremove-cached.py $P-tcltk
EOF

	tar -cjf $R/$P/$P-tcltk/$P-tcltk-$V-$B.tar.bz2 \
		--xform "s,tcltk-preremove.bat,etc/setup/$P-tcltk.bat," \
		--xform "s,$a/,$PREFIX," \
		--exclude "__pycache__" \
		--exclude-from "test.lst" \
		-T tcltk.lst \
		tcltk-preremove.bat

	#
	# tools
	#

	cat <<EOF >$R/$P/$P-tools/setup.hint
sdesc: "Python Tools"
ldesc: "Python Tools"
maintainer: $MAINTAINER
category: Commandline_Utilities
requires: $P-core
external-source: $P-core
EOF

        cat <<EOF >tools-preremove.bat
call %OSGEO4W_ROOT%\\bin\\py3_env.bat
python -B %OSGEO4W_ROOT%\\apps\\Python$MM\\Scripts\\preremove-cached.py $P-tools
EOF

	tar -cjf $R/$P/$P-tools/$P-tools-$V-$B.tar.bz2 \
		--xform "s,tools-preremove.bat,etc/setup/$P-tools.bat," \
		--xform "s,$a/,$PREFIX," \
		--exclude "__pycache__" \
		--exclude-from tcltk.lst \
		--exclude-from test.lst \
		$a/Tools \
		tools-preremove.bat

	#
	# pip
	#

	rm -f pip-postinstall.bat

	cat <<EOF >pip-preremove.bat
call %OSGEO4W_ROOT%\\bin\\py3_env.bat
python -B %OSGEO4W_ROOT%\\apps\\Python$MM\\Scripts\\preremove-cached.py $P-pip
EOF

	exetmpl $a/Scripts/pip.exe pip
	exetmpl $a/Scripts/pip$M.exe pip
	exetmpl $a/Scripts/pip$VM.exe pip


	cat <<EOF >$R/$P/$P-pip/setup.hint
sdesc: "The PyPA recommended tool for installing Python packages."
ldesc: "The PyPA recommended tool for installing Python packages."
maintainer: $MAINTAINER
category: Libs
requires: $P-core
EOF

	tar -cjf $R/$P/$P-pip/$P-pip-$PIPV-$B.tar.bz2 \
		--xform "s,^pip-postinstall.bat,etc/postinstall/$P-pip.bat," \
		--xform "s,^pip-preremove.bat,etc/preremove/$P-pip.bat," \
		--xform s,^./pip.exe.tmpl,${PREFIX}Scripts/pip.exe.tmpl, \
		--xform s,^./pip$M.exe.tmpl,${PREFIX}Scripts/pip$M.exe.tmpl, \
		--xform s,^./pip$VM.exe.tmpl,${PREFIX}Scripts/pip$VM.exe.tmpl, \
		--xform "s,$a/,$PREFIX," \
		--exclude "__pycache__" \
		-T pip.lst \
		pip-postinstall.bat \
		pip-preremove.bat

	#
	# setuptools
	#

	rm -f setuptools-postinstall.bat

	cat <<EOF >setuptools-preremove.bat
call %OSGEO4W_ROOT%\\bin\\py3_env.bat
python -B %OSGEO4W_ROOT%\\apps\\Python$MM\\Scripts\\preremove-cached.py $P-setuptools
EOF

	exetmpl $a/Scripts/easy_install-$VM.exe setuptools
	exetmpl $a/Scripts/easy_install.exe setuptools

	cat <<EOF >$R/$P/$P-setuptools/setup.hint
sdesc: "setuptools - Easily download, build, install, upgrade, and uninstall Python packages"
ldesc: "setuptools - Easily download, build, install, upgrade, and uninstall Python packages"
maintainer: $MAINTAINER
category: Libs
requires: $P-core
EOF
	tar -cjf $R/$P/$P-setuptools/$P-setuptools-$STV-$B.tar.bz2 \
		--xform "s,^setuptools-postinstall.bat,etc/postinstall/$P-setuptools.bat," \
		--xform "s,^setuptools-preremove.bat,etc/preremove/$P-setuptools.bat," \
		--xform "s,^easy_install-$VM.exe.tmpl,${PREFIX}Scripts/easy_install-$VM.exe.tmpl," \
		--xform "s,^easy_install.exe.tmpl,${PREFIX}Scripts/easy_install.exe.tmpl," \
		--xform "s,$a/,$PREFIX," \
		--exclude "__pycache__" \
		setuptools-postinstall.bat \
		setuptools-preremove.bat \
		-T setuptools.lst

	tar -C .. -cjf $R/$P/$P-core/$P-core-$V-$B-src.tar.bz2 \
		osgeo4w/package.sh
	
	(
		tar -tjf $R/$P/$P-core/$P-core-$V-$B.tar.bz2 | sed -e "s/$/:core/"
		tar -tjf $R/$P/$P-help/$P-help-$V-$B.tar.bz2 | sed -e "s/$/:help/"
		tar -tjf $R/$P/$P-devel/$P-devel-$V-$B.tar.bz2 | sed -e "s/$/:devel/"
		tar -tjf $R/$P/$P-test/$P-test-$V-$B.tar.bz2 | sed -e "s/$/:test/"
		tar -tjf $R/$P/$P-tcltk/$P-tcltk-$V-$B.tar.bz2 | sed -e "s/$/:tcltk/"
		tar -tjf $R/$P/$P-tools/$P-tools-$V-$B.tar.bz2 | sed -e "s/$/:tools/"
		tar -tjf $R/$P/$P-pip/$P-pip-$PIPV-$B.tar.bz2 | sed -e "s/$/:pip/"
		tar -tjf $R/$P/$P-setuptools/$P-setuptools-$STV-$B.tar.bz2 | sed -e "s/$/:setuptools/"
	) | fgrep -v "/:" | sort >/tmp/$P-$a.lst

	cut -d: -f1 /tmp/$P-$a.lst | sort | uniq -d >/tmp/$P-$a.dupes

	if [ -s /tmp/$P-$a.dupes ]; then
		echo DUPES:
		fgrep -f /tmp/$P-$a.dupes /tmp/$P-$a.lst | sort
		break
	fi
done
